@@ -90,7 +90,7 @@ namespace sdbus::internal {
         virtual int sd_bus_process(sd_bus *bus, sd_bus_message **r) = 0;
         virtual sd_bus_message* sd_bus_get_current_message(sd_bus *bus) = 0;
         virtual int sd_bus_get_poll_data(sd_bus *bus, PollData* data) = 0;
-        virtual int sd_bus_get_n_queued_read(sd_bus *bus, uint64_t *ret) = 0;
+        virtual int sd_bus_get_n_queued(sd_bus *bus, uint64_t *read, uint64_t* write) = 0;
         virtual int sd_bus_flush(sd_bus *bus) = 0;
         virtual sd_bus *sd_bus_flush_close_unref(sd_bus *bus) = 0;
         virtual sd_bus *sd_bus_close_unref(sd_bus *bus) = 0;
@@ -98,6 +98,7 @@ namespace sdbus::internal {
         virtual int sd_bus_message_set_destination(sd_bus_message *m, const char *destination) = 0;
 
         virtual int sd_bus_query_sender_creds(sd_bus_message *m, uint64_t mask, sd_bus_creds **c) = 0;
+        virtual sd_bus_creds* sd_bus_creds_ref(sd_bus_creds *c) = 0;
         virtual sd_bus_creds* sd_bus_creds_unref(sd_bus_creds *c) = 0;
 
         virtual int sd_bus_creds_get_pid(sd_bus_creds *c, pid_t *pid) = 0;
