@@ -97,6 +97,29 @@ TEST(AVariant, CanBeCopied)
     ASSERT_THAT(variantCopy2.get<std::string>(), Eq(value));
 }
 
+TEST(AVariant, CanBeMoved)
+{
+    auto value = "hello"s;
+    sdbus::Variant variant(value);
+
+    auto movedVariant{std::move(variant)};
+
+    ASSERT_THAT(movedVariant.get<std::string>(), Eq(value));
+    ASSERT_TRUE(variant.isEmpty());
+}
+
+TEST(AVariant, CanBeMovedIntoAMap)
+{
+    auto value = "hello"s;
+    sdbus::Variant variant(value);
+
+    std::map<std::string, sdbus::Variant> mymap;
+    mymap.try_emplace("payload", std::move(variant));
+
+    ASSERT_THAT(mymap["payload"].get<std::string>(), Eq(value));
+    ASSERT_TRUE(variant.isEmpty());
+}
+
 TEST(AVariant, IsNotEmptyWhenContainsAValue)
 {
     sdbus::Variant v("hello");
@@ -322,7 +345,6 @@ TEST(ASignature, CanBeMovedLikeAStdString)
     sdbus::Signature oSignature{aSignature};
 
     ASSERT_THAT(sdbus::Signature{std::move(oSignature)}, Eq(sdbus::Signature(std::move(aSignature))));
-    ASSERT_THAT(std::string(oSignature), Eq(aSignature));
 }
 
 TEST(AUnixFd, DuplicatesAndOwnsFdUponStandardConstruction)
@@ -428,18 +450,60 @@ TEST(AUnixFd, TakesOverNewFdAndClosesOriginalFdOnAdoptingReset)
 
 TEST(AnError, CanBeConstructedFromANameAndAMessage)
 {
-    auto error = sdbus::Error("name", "message");
-    EXPECT_THAT(error.getName(), Eq<std::string>("name"));
+    auto error = sdbus::Error(sdbus::Error::Name{"org.sdbuscpp.error"}, "message");
+    EXPECT_THAT(error.getName(), Eq<std::string>("org.sdbuscpp.error"));
     EXPECT_THAT(error.getMessage(), Eq<std::string>("message"));
+    EXPECT_TRUE(error.isValid());
 }
 
 TEST(AnError, CanBeConstructedFromANameOnly)
 {
-    auto error1 = sdbus::Error("name");
-    auto error2 = sdbus::Error("name", nullptr);
-    EXPECT_THAT(error1.getName(), Eq<std::string>("name"));
-    EXPECT_THAT(error2.getName(), Eq<std::string>("name"));
+    auto error1 = sdbus::Error(sdbus::Error::Name{"org.sdbuscpp.error"});
+    auto error2 = sdbus::Error(sdbus::Error::Name{"org.sdbuscpp.error"}, nullptr);
+    EXPECT_THAT(error1.getName(), Eq<std::string>("org.sdbuscpp.error"));
+    EXPECT_THAT(error2.getName(), Eq<std::string>("org.sdbuscpp.error"));
 
-    EXPECT_THAT(error1.getMessage(), Eq<std::string>(""));
-    EXPECT_THAT(error2.getMessage(), Eq<std::string>(""));
+    EXPECT_TRUE(error1.getMessage().empty());
+    EXPECT_TRUE(error2.getMessage().empty());
+
+    EXPECT_TRUE(error1.isValid());
+    EXPECT_TRUE(error2.isValid());
+}
+
+TEST(AnError, IsInvalidWhenConstructedWithAnEmptyName)
+{
+    auto error = sdbus::Error({});
+
+    EXPECT_TRUE(error.getName().empty());
+    EXPECT_TRUE(error.getMessage().empty());
+    EXPECT_FALSE(error.isValid());
+}
+
+TEST(AnErrorFactory, CanCreateAnErrorFromErrno)
+{
+    auto error = sdbus::createError(ENOENT, "custom message");
+
+    EXPECT_THAT(error.getName(), Eq<std::string>("org.freedesktop.DBus.Error.FileNotFound"));
+    EXPECT_THAT(error.getMessage(), Eq<std::string>("custom message (No such file or directory)"));
+    EXPECT_TRUE(error.isValid());
+}
+
+#ifndef SDBUS_basu // Creating error from invalid errno is not supported on basu backend
+TEST(AnErrorFactory, CreatesGenericErrorWhenErrnoIsUnknown)
+{
+    auto error = sdbus::createError(123456, "custom message");
+
+    EXPECT_THAT(error.getName(), Eq<std::string>("org.freedesktop.DBus.Error.Failed"));
+    EXPECT_THAT(error.getMessage(), Eq<std::string>("custom message (Unknown error 123456)"));
+    EXPECT_TRUE(error.isValid());
+}
+#endif // SDBUS_basu
+
+TEST(AnErrorFactory, CreatesEmptyInvalidErrorWhenErrnoIsZero)
+{
+    auto error = sdbus::createError(0, "custom message");
+
+    EXPECT_TRUE(error.getName().empty());
+    EXPECT_THAT(error.getMessage(), Eq<std::string>("custom message"));
+    EXPECT_FALSE(error.isValid());
 }
